==> str_to_obj-2024.7.tar.gz: str_to_obj-2024.7/package/str_to_obj/main.py <==
"""
Copyright CNRS/Inria/UniCA
Contributor(s): Eric Debreuve (eric.debreuve@cnrs.fr) since 2023
SEE COPYRIGHT NOTICE BELOW
"""

import ast as bstr
import types as t
import typing as h

from str_to_obj.task.casting import CastValue
from str_to_obj.type.hint_tree import any_hint_h, hint_t


def ObjectFromStr(
    string: str,
    /,
    *,
    expected_type: any_hint_h | hint_t | type[t.NoneType] = None,
) -> tuple[h.Any, list[str]]:
    """
    expected_type: Must not be passed explicitly as None since None is interpreted as
    "no specific expected type". When expecting None, pass NoneType.
    """
    if expected_type is t.NoneType:
        if string.lower() == "none":
            issues = []
        else:
            issues = [f'{string}: Invalid value; Expected="NONE" or "None" or "none".']
        return None, issues

    value = _ObjectFromStr(string)

    if expected_type is None:
        return value, []

    return CastValue(value, expected_type)


def _ObjectFromStr(string:str, /)-> h.Any:
    """"""
    try:
        value = bstr.literal_eval(string)
    except (SyntaxError, ValueError):
        value = string

    if isinstance(value, h.Sequence) and not isinstance(value, str):
        stripe = type(value)
        value = stripe(_ObjectFromStr(_elm) for _elm in value)

    return value


"""
COPYRIGHT NOTICE

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

SEE LICENCE NOTICE: file README-LICENCE-utf8.txt at project source root.

This software is being developed by Eric Debreuve, a CNRS employee and
member of team Morpheme.
Team Morpheme is a joint team between Inria, CNRS, and UniCA.
It is hosted by the Centre Inria d'Université Côte d'Azur, Laboratory
I3S, and Laboratory iBV.

CNRS: https://www.cnrs.fr/index.php/en
Inria: https://www.inria.fr/en/
UniCA: https://univ-cotedazur.eu/
Centre Inria d'Université Côte d'Azur: https://www.inria.fr/en/centre/sophia/
I3S: https://www.i3s.unice.fr/en/
iBV: http://ibv.unice.fr/
Team Morpheme: https://team.inria.fr/morpheme/
"""

==> str_to_obj-2025.3.tar.gz: str_to_obj-2025.3/package/str_to_obj/main.py <==
"""
Copyright CNRS/Inria/UniCA
Contributor(s): Eric Debreuve (eric.debreuve@cnrs.fr) since 2023
SEE COPYRIGHT NOTICE BELOW
"""

import ast as bstr
import types as t
import typing as h

from str_to_obj.task.casting import CastValue
from str_to_obj.type.hint_tree import any_hint_h, hint_t


def ObjectFromStr(
    string: str,
    /,
    *,
    expected_type: any_hint_h | hint_t | type[t.NoneType] = None,
) -> tuple[h.Any, list[str]]:
    """
    expected_type: Must not be passed explicitly as None since None is interpreted as
    "no specific expected type". When expecting None, pass NoneType.
    """
    if expected_type is t.NoneType:
        if string.lower() == "none":
            issues = []
        else:
            issues = [f'{string}: Invalid value; Expected="NONE" or "None" or "none".']
        return None, issues

    value = _ObjectFromStr(string)

    if expected_type is None:
        return value, []

    return CastValue(value, expected_type)


def _ObjectFromStr(string: str | h.Any, /)-> h.Any:
    """"""
    if not isinstance(string, str):
        return string

    try:
        value = bstr.literal_eval(string)
    except (SyntaxError, ValueError):
        value = string

    if isinstance(value, h.Sequence) and not isinstance(value, str):
        stripe = type(value)
        value = stripe(_ObjectFromStr(_elm) for _elm in value)

    return value


"""
COPYRIGHT NOTICE

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

SEE LICENCE NOTICE: file README-LICENCE-utf8.txt at project source root.

This software is being developed by Eric Debreuve, a CNRS employee and
member of team Morpheme.
Team Morpheme is a joint team between Inria, CNRS, and UniCA.
It is hosted by the Centre Inria d'Université Côte d'Azur, Laboratory
I3S, and Laboratory iBV.

CNRS: https://www.cnrs.fr/index.php/en
Inria: https://www.inria.fr/en/
UniCA: https://univ-cotedazur.eu/
Centre Inria d'Université Côte d'Azur: https://www.inria.fr/en/centre/sophia/
I3S: https://www.i3s.unice.fr/en/
iBV: http://ibv.unice.fr/
Team Morpheme: https://team.inria.fr/morpheme/
"""
